@@ -11,11 +11,16 @@ import boto3
 import numpy as np
 import pandas as pd
 import ujson
+from pyquaternion import Quaternion
 from PIL import Image, ImageEnhance
 from scaleapi import Task
+import matplotlib.pyplot as plt
+from matplotlib.widgets import Slider, Button
+import matplotlib.gridspec as gridspec
 
 from .camera import LidarCamera
 from .transform import Transform
+from .connectors import Importer
 
 UPLOAD_POOL_SIZE = 8
 s3 = boto3.client('s3')
@@ -149,6 +154,9 @@ class LidarFrame:
             self.images[camera_id] = LidarImage(camera=self.cameras[camera_id])
         return self.images[camera_id]
 
+    def add_points_from_connector(self, connector: Importer, transform: Transform = None, intensity=1, sensor_id=0):
+        self.add_points(connector.data, transform, intensity, sensor_id)
+
     def add_points(self, points: np.array, transform: Transform = None, intensity=1, sensor_id=0):
         if points.ndim == 1:
             points = np.array([points])
@@ -169,10 +177,59 @@ class LidarFrame:
     def get_world_points(self):
         return np.hstack([self.transform @ self.points[:, :3], self.points[:, 3:4], self.points[:, 4:5]])
 
+    # leeaving this method as legacy / old code dependency
     def get_projected_image(self, camera_id, color_mode='default', **kwargs):
-        return self.cameras[camera_id].get_projected_image(self.get_image(camera_id), self.points, self.transform,
-                                                           color_mode, **kwargs)
+        return self.cameras[camera_id].get_projected_image(self.get_image(camera_id), self.points, self.transform, color_mode, **kwargs)
 
+    def manual_calibration(self, camera_id, intrinsics_ratio=1000, extrinsics_ratio=10):
+        fig = plt.figure(constrained_layout=True)
+        intrinsics = ['fx','fy','cx', 'cy']
+        extrinsics_position = ['x','y','z']
+        extrinsics_heading = ['qw','qx','qy','qz']
+        heights = np.concatenate((np.array([7]), np.array([0.2 for x in range(0, 7)])), axis=0)
+        gs = fig.add_gridspec(ncols= 3, nrows=1 + len(extrinsics_heading) + len(extrinsics_position), height_ratios=heights)
+        imgObj = fig.add_subplot(gs[0, :])
+        imgObj.imshow(self.cameras[camera_id].get_projected_image(self.get_image(camera_id), self.points, self.transform, 'depth', 1))
+
+        for index, key in enumerate(intrinsics):
+            globals()[f"ax{key}"] = fig.add_subplot(gs[index + 1, 0])
+            value = getattr(self.cameras[camera_id], key)
+            globals()[key] = Slider(globals()[f"ax{key}"], f"{key}", value-intrinsics_ratio, value+intrinsics_ratio, valinit=value)
+
+        for index, key in enumerate(extrinsics_position):
+            globals()[f"ax{key}"] = fig.add_subplot(gs[index + 1, 1])
+            value = getattr(self.cameras[camera_id], 'position')
+            globals()[key] = Slider(globals()[f"ax{key}"], f"{key}", value[index]-extrinsics_ratio, value[index]+extrinsics_ratio, valinit=value[index])
+
+        for index, key in enumerate(extrinsics_heading):
+            globals()[f"ax{key}"] = fig.add_subplot(gs[index + len(extrinsics_position) + 1, 1])
+            value = getattr(self.cameras[camera_id], 'rotation')
+            value = Quaternion(matrix=value)
+            globals()[key] = Slider(globals()[f"ax{key}"], f"{key}", value[index]-extrinsics_ratio, value[index]+extrinsics_ratio, valinit=value[index])
+
+        def update(val):
+            self.cameras[camera_id].calibrate(K=np.array([
+                [fx.val, 0, cx.val],
+                [0, fy.val, cy.val],
+                [0,0,1]
+                ]),
+                pose= Transform.from_Rt(
+                    Quaternion(qw.val,qx.val,qy.val,qz.val).rotation_matrix,
+                    [x.val,y.val,z.val]
+                    )
+                )
+            np.set_printoptions(suppress=True)
+            print(f"New intrinsics for Camera {camera_id}")
+            print(self.cameras[camera_id].K)
+            print(f"New position for Camera {camera_id}")
+            print(self.cameras[camera_id].position)
+            imgObj.imshow(self.cameras[camera_id].get_projected_image(self.get_image(camera_id), self.points, self.transform, 'depth', 1))
+            fig.canvas.draw_idle()
+
+        axApply = fig.add_subplot(gs[1, 2])
+        bApply = Button(axApply, 'Apply changes')
+        bApply.on_clicked(update)
+        plt.show()
     def get_filename(self):
         return "frame-%s.json" % self.id
 
@@ -199,11 +256,14 @@ class LidarFrame:
                 fy=camera.fy,
                 cx=camera.cx,
                 cy=camera.cy,
+                skew=camera.skew,
                 k1=float(camera.D[0]),
                 k2=float(camera.D[1]),
                 p1=float(camera.D[2]),
                 p2=float(camera.D[3]),
                 k3=float(camera.D[4]),
+                k4=float(camera.D[5]),
+                scale_factor=camera.scale_factor
             )
             if image.metadata:
                 result['metadata'] = image.metadata
@@ -310,6 +370,15 @@ class LidarScene:
         for frame in self.frames:
             frame.filter_points(min_intensity, min_intensity_percentile)
 
+    def get_projected_image(self, camera_id, color_mode='intensity', frames_index=range(0,1), **kwargs):
+        all_points = np.array([]).reshape(0,4)
+        for idx in frames_index:
+            points = np.array(self.frames[idx].points)[:,:4]
+            print(points.shape)
+            points[:, 3] = idx
+            all_points = np.concatenate((all_points, points), axis=0)
+        return self.cameras[camera_id].get_projected_image(self.frames[frames_index[0]].get_image(camera_id), all_points, self.frames[frames_index[0]].transform, color_mode)
+
     def apply_transform(self, world_transform: Transform):
         for idx in range(len(self.frames)):
             self.get_frame(index=idx).apply_transform(world_transform)
@@ -376,7 +445,7 @@ class LidarScene:
         print('Saving scene:', filepath)
         with zipfile.ZipFile(filepath, mode='w') as out:
             # Save task
-            scene_dict = self.to_dict('')
+            scene_dict = self.to_dict()
             task_dict = dict(
                 template or {},
                 attachment_type='json',
@@ -387,7 +456,7 @@ class LidarScene:
             # Save frames
             for frame in self.frames:
                 # Save points
-                data = frame.to_json('')
+                data = frame.to_json(self.public_url)
                 out.writestr('frame-%s.json' % frame.id, data, compress_type=zipfile.ZIP_DEFLATED)
 
                 # Save frame images
@@ -413,5 +482,8 @@ class LidarScene:
         elif task_type == 'lidartopdown':
             from .task import LidarTopDownTask
             return LidarTopDownTask.from_scene(self, template)
+        elif task_type == 'lidarsegmentation':
+            from .task import LidarSegmentation
+            return LidarSegmentation.from_scene(self, template)
         else:
             assert NotImplemented(f'Specified task_type {task_type} is not supported')
